use wccg_models::event::Scorecard;
use wread_data_mongodb::crud_repository;
use wread_data_mongodb::mongodb::bson::{doc, oid::ObjectId};
use wread_data_mongodb::mongodb::error::Error;
use wread_data_mongodb::mongodb::results::{DeleteResult, InsertOneResult, UpdateResult};
use wread_data_mongodb::mongodb::Database;

const COLLECTION_NAME: &str = "scorecards";

pub async fn get(
    id: &str,
    db: &Database,
) -> Result<Option<Scorecard>, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    crud_repository::find_by_id(&id_object, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id(
    rider_id: &str,
    db: &Database,
) -> Result<Vec<Scorecard>, Error> {
    let filter = doc! {"riderId": rider_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_event_id(
    event_id: &str,
    db: &Database,
) -> Result<Vec<Scorecard>, Error> {
    let filter = doc! {"eventId": event_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_event(
    rider_id: &str,
    event_id: &str,
    db: &Database,
) -> Result<Vec<Scorecard>, Error> {
    let filter = doc! {"riderId": rider_id, "eventId": event_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_registration(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Option<Scorecard>, Error> {
    let filter = doc! {"riderId": rider_id, "eventId": event_id, "registrationOptionId": registration_option_id };
    crud_repository::find_one(filter, COLLECTION_NAME, &db).await
}

pub async fn add(summary: &Scorecard, db: &Database) -> Result<InsertOneResult, Error> {
    crud_repository::add(summary, COLLECTION_NAME, db).await
}

pub async fn delete(id: &str, db: &Database) -> Result<DeleteResult, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    let query = doc! {"_id": id_object };
    crud_repository::delete_one(query, None, COLLECTION_NAME, &db).await
}

pub async fn update(summary: Scorecard, db: &Database) -> Result<UpdateResult, Error> {
    let id = summary.id().clone().unwrap();
    let filter = doc! {"_id":  id};
    crud_repository::replace_one(filter, &summary, None, COLLECTION_NAME, db).await
}
